/*
 *                            COPYRIGHT
 *
 *  cschem - modular/flexible schematics editor - sch-rnd (executable)
 *  Copyright (C) 2021 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/cschem
 *    lead developer: email to cschem (at) igor2.repo.hu
 *    mailing list: cschem (at) list.repo.hu (send "subscribe")
 */

#include <libcschem/config.h>
#include <libcschem/concrete.h>
#include <libcschem/operation.h>
#include <libcschem/cnc_obj.h>
#include <libcschem/search.h>

#include "search.h"

/* Return the preferred visible object at x;y or NULL if nothing found */
csch_chdr_t *sch_rnd_search_obj_at(csch_sheet_t *sheet, csch_coord_t x, csch_coord_t y, csch_coord_t radius)
{
	csch_rtree_box_t box;

	box.x1 = x - radius; box.y1 = y - radius;
	box.x2 = x + radius; box.y2 = y + radius;

	return csch_search_first_gui(sheet, &box);
}


void sch_rnd_search_obj_box(csch_sheet_t *sheet, vtp0_t *res, csch_rtree_box_t *box)
{
	int n;

	for(n = 0; n < CSCH_DSPLY_max; n++) {
		csch_chdr_t *o;
		csch_rtree_it_t it;

		if (!csch_layer_vis[n])
			continue;

		/* prefere/find unlocked object first */
		for(o = csch_rtree_first(&it, &sheet->dsply[n], box); o != NULL; o = csch_rtree_next(&it)) {
			if (csch_cobj_is_locked(o))
				continue;
			if (csch_isc_with_box(o, box))
				vtp0_append(res, o);
		}


		/* fall back to parent of locked objects (e.g. clicked a group-locked
		   object, return group) */
		for(o = csch_rtree_first(&it, &sheet->dsply[n], box); o != NULL; o = csch_rtree_next(&it)) {
			csch_chdr_t *cand = csch_cobj_first_unlocked(o);
			if ((cand != NULL) && csch_isc_with_box(cand, box))
				vtp0_append(res, cand);
		}

	}
}
